import AsyncStorage from '@react-native-async-storage/async-storage';
import { withMemoryCache } from './memory';
import { KeyPrefixHelper, defaultPrefix } from './utils';
class AsyncStorageCache {
    constructor(keyPrefix) {
        this.prefixHelper = new KeyPrefixHelper(keyPrefix);
    }
    get(key) {
        if (this.keys && !this.keys.has(key)) {
            return Promise.resolve(undefined);
        }
        return AsyncStorage.getItem(this.prefixHelper.appendPrefix(key)).then((strValue) => {
            if (!strValue) {
                return Promise.resolve(undefined);
            }
            try {
                const value = JSON.parse(strValue);
                return Promise.resolve(value);
            }
            catch (err) {
                // failed to parse, do nothing
            }
            return Promise.resolve(undefined);
        });
    }
    set(key, value) {
        const strValue = JSON.stringify(value);
        return AsyncStorage.setItem(this.prefixHelper.appendPrefix(key), strValue).then(() => {
            var _a;
            (_a = this.keys) === null || _a === void 0 ? void 0 : _a.add(key);
        });
    }
    remove(key) {
        return AsyncStorage.removeItem(this.prefixHelper.appendPrefix(key)).then(() => {
            var _a;
            (_a = this.keys) === null || _a === void 0 ? void 0 : _a.delete(key);
        });
    }
    getKeys() {
        if (this.keys) {
            return Promise.resolve(Array.from(this.keys));
        }
        // search from storage if there is no local cache
        return AsyncStorage.getAllKeys().then((keys) => {
            const matched = keys
                .filter((key) => this.prefixHelper.matchPrefix(key))
                .map((key) => this.prefixHelper.removePrefix(key));
            this.keys = new Set(matched);
            return matched;
        });
    }
    clear() {
        return this.getKeys().then((keys) => {
            var _a;
            (_a = this.keys) === null || _a === void 0 ? void 0 : _a.clear();
            return AsyncStorage.multiRemove(keys.map((key) => this.prefixHelper.appendPrefix(key)));
        });
    }
}
export function createAsyncStorageCache(keyPrefix = defaultPrefix, useMemoryCache = true) {
    const cache = new AsyncStorageCache(keyPrefix);
    return useMemoryCache ? withMemoryCache(cache) : cache;
}
