import { withMemoryCache } from './memory';
import { KeyPrefixHelper, defaultPrefix } from './utils';
class LocalStorageCache {
    constructor(keyPrefix) {
        this.prefixHelper = new KeyPrefixHelper(keyPrefix);
    }
    get(key) {
        if (this.keys && !this.keys.has(key)) {
            return Promise.resolve(undefined);
        }
        const strValue = localStorage.getItem(this.prefixHelper.appendPrefix(key));
        if (!strValue) {
            return Promise.resolve(undefined);
        }
        try {
            const value = JSON.parse(strValue);
            return Promise.resolve(value);
        }
        catch (err) {
            // failed to parse, do nothing
        }
        return Promise.resolve(undefined);
    }
    set(key, value) {
        var _a;
        const strValue = JSON.stringify(value);
        localStorage.setItem(this.prefixHelper.appendPrefix(key), strValue);
        (_a = this.keys) === null || _a === void 0 ? void 0 : _a.add(key);
        return Promise.resolve();
    }
    remove(key) {
        var _a;
        localStorage.removeItem(this.prefixHelper.appendPrefix(key));
        (_a = this.keys) === null || _a === void 0 ? void 0 : _a.delete(key);
        return Promise.resolve();
    }
    getKeys() {
        if (!this.keys) {
            this.keys = new Set();
            // search from localStorage if there is no local cache
            for (let i = 0; i < localStorage.length; i++) {
                const key = localStorage.key(i);
                if (this.prefixHelper.matchPrefix(key)) {
                    this.keys.add(this.prefixHelper.removePrefix(key));
                }
            }
        }
        return Promise.resolve(Array.from(this.keys));
    }
    clear() {
        return this.getKeys().then((keys) => {
            var _a;
            (_a = this.keys) === null || _a === void 0 ? void 0 : _a.clear();
            return Promise.all(keys.map((key) => this.remove(key)));
        });
    }
}
export function createLocalStorageCache(keyPrefix = defaultPrefix, useMemoryCache = true) {
    const cache = new LocalStorageCache(keyPrefix);
    return useMemoryCache ? withMemoryCache(cache) : cache;
}
