import { Fetcher, RequestOptions } from '../types';
export interface PaginationListOptions extends Omit<RequestOptions, 'pollingWaitTime'> {
    /**
     * start the request by manual call `refresh()` or `loadMore()`, default is false
     */
    manualStart?: boolean;
}
export interface PaginationListState<L, T> {
    /**
     * the concat list
     */
    list: L[];
    /**
     * latest response data
     */
    data?: T;
    /**
     * latest error
     */
    error?: Error;
    /**
     * is loading data - true for both refreshing and loading more
     */
    isLoading: boolean;
    /**
     * is refreshing data - true for refreshing
     */
    isRefreshing: boolean;
    /**
     * there is still more list items to load or not
     */
    hasMore: boolean;
    /**
     *
     */
    loadMore: () => void;
    /**
     * refresh list data
     */
    refresh: () => void;
}
/**
 * extract list and create next request from received data + previous request
 */
export declare type DataHandler<L, T, R> = (data: T, prevRequest?: R) => {
    /**
     * list extracted from response data
     */
    list: L[];
    /**
     * next request, null if it reaches end of the list
     */
    nextRequest: R | null;
};
/**
 *
 * @param fetcher
 * @param listExtractor
 * @param nextRequestCreator
 * @param initialRequest
 * @param options
 */
export declare function usePaginationList<L, T, R>(fetcher: Fetcher<T, R>, dataHandler: DataHandler<L, T, R>, initialRequest?: R, options?: PaginationListOptions): PaginationListState<L, T>;
