import { Fetcher, RequestOptions } from '../types';
export interface PollingOptions extends Omit<RequestOptions, 'pollingWaitTime'> {
    /**
     * start the polling by manual call `start()` (don't auto start), default is false
     */
    manualStart?: boolean;
}
export interface PollingState<T> {
    data?: T;
    error?: Error;
    /**
     * polling is started or not
     */
    isPolling: boolean;
    /**
     * start polling
     */
    start: () => void;
    /**
     * stop polling
     */
    stop: () => void;
}
/**
 * Send polling requests
 * @param url url used for polling
 * @param pollingWaitTime wait time between polling requests in seconds
 * @param request fetch() options
 * @param options polling
 */
export declare function usePolling<T>(url: string, pollingWaitTime: number, request?: RequestInit, options?: PollingOptions): PollingState<T>;
/**
 * Send polling requests
 * @param fetcher fetcher used for polling requests
 * @param pollingWaitTime wait time between polling requests in seconds
 * @param request request params
 * @param options polling options
 */
export declare function usePolling<T, R = void>(fetcher: Fetcher<T, R>, pollingWaitTime: number, request?: R, options?: PollingOptions): any;
