import { useEffect, useRef, useCallback } from 'react';
import { forEachResponse } from '../utils';
import { getURLFetcher } from './fetcher';
import { useDeepEqualMemo, useRerender, useShallowEqualMemo } from './utils';
// implementation
export function usePolling(fetcherOrURL, pollingWaitTime, request, options) {
    const fetcher = typeof fetcherOrURL === 'string' ? getURLFetcher(fetcherOrURL) : fetcherOrURL;
    const requestMemo = useDeepEqualMemo(request);
    const optionsMemo = useShallowEqualMemo(options);
    // use ref to keep current state
    const stateRef = useRef({
        isPolling: false,
    });
    const rerender = useRerender();
    const abortRef = useRef();
    // stop polling control
    const stop = useCallback(() => {
        var _a;
        if (!stateRef.current.isPolling) {
            return;
        }
        (_a = abortRef.current) === null || _a === void 0 ? void 0 : _a.call(abortRef);
        stateRef.current = Object.assign(Object.assign({}, stateRef.current), { isPolling: false });
        rerender();
    }, []);
    stateRef.current.stop = stop;
    const start = useCallback(() => {
        if (stateRef.current.isPolling) {
            return;
        }
        abortRef.current = forEachResponse(fetcher.fetch(requestMemo, Object.assign({}, optionsMemo, {
            pollingWaitTime,
        })), ({ data, error }) => {
            stateRef.current = Object.assign(Object.assign({}, stateRef.current), { data: data !== null && data !== void 0 ? data : stateRef.current.data, error });
            rerender();
        });
        stateRef.current = Object.assign(Object.assign({}, stateRef.current), { error: undefined, isPolling: true });
        rerender();
    }, [rerender, fetcher, pollingWaitTime, requestMemo, optionsMemo]);
    stateRef.current.start = start;
    useEffect(() => {
        // auto start
        const { manualStart } = optionsMemo !== null && optionsMemo !== void 0 ? optionsMemo : {};
        if (!manualStart) {
            start();
        }
        return () => {
            // make sure polling stop if any params updated
            stop();
        };
    }, [start, stop, optionsMemo]);
    return stateRef.current;
}
