import { Fetcher, RequestOptions, CacheMode } from '../types';
export interface SWROptions extends Omit<RequestOptions, 'pollingWaitTime'> {
    /**
     * start the fetch request by manual call `refresh()` (don't auto start), default is false
     */
    manualStart?: boolean;
}
export interface SWRState<T> {
    data?: T;
    error?: Error;
    /**
     * is initial data loaded, it may come from cache
     */
    isLoaded: boolean;
    /**
     * is data fresh or validated by sending request
     */
    isFreshOrValidated: boolean;
    /**
     * refresh data by trigger another swr request process
     * @param cacheMode optional override cacheMode for this refresh fetch
     */
    refresh: (cacheMode?: CacheMode) => void;
}
/**
 * Send request and return data in SWR way
 * @param url url to fetch
 * @request fetch() options
 * @options swr options
 */
export declare function useSWR<T>(url: string, request?: RequestInit, options?: SWROptions): SWRState<T>;
/**
 * Send request and return data in SWR way
 * @param fetcher fetcher used
 * @param request request params
 * @param options swr options
 */
export declare function useSWR<T, R = void>(fetcher: Fetcher<T, R>, request?: R, options?: SWROptions): SWRState<T>;
