import { RequestResponse, RequestReturn, Fetcher, Cache } from './types';
export declare type ResponseHandler<T> = (response: RequestResponse<T>) => void;
/**
 * helper for handle each fetcher.fetch() response
 * @param requestReturn the fetcher.fetch() request return
 * @param handler handle that deal with each response
 * @returns abort function that can abort this request
 */
export declare function forEachResponse<T>(requestReturn: RequestReturn<T>, handler: ResponseHandler<T>): () => void;
/**
 * helper for handle each fetcher.fetch() response
 * and returns the latest data it receives after all responses settled
 * NOTE:
 *  - it only reject with error if there is no any data available
 *  - this method SHOULD NOT be used together with polling requests - which never ends
 * @param requestReturn the fetcher.fetch() request return
 */
export declare function getFinalResponse<T>(requestReturn: RequestReturn<T>): Promise<T>;
/**
 * helper for handle each fetcher.fetch() response
 * and returns the initial data it receives no matter it comes from valid cache or network
 * NOTE:
 *  - it only reject with error if there is no any data available
 *  - this method SHOULD NOT be used together with polling requests - which never ends
 * @param requestReturn the fetcher.fetch() request return
 */
export declare function getInitialResponse<T>(requestReturn: RequestReturn<T>): Promise<T>;
export declare type PureFetch<T, R = void> = (request?: R) => Promise<T>;
/**
 * fallback to a pure fetch function, which uses no cache store and return plain promise wrapped response
 * @param fetcher
 */
export declare function fallbackToPureFetch<T, R = void>(fetcher: Fetcher<T, R>): PureFetch<T, R>;
/**
 * clear cache
 * @param cache cache to be cleared
 * @param maxAge only keys older than max age will be cleared if maxAge set
 */
export declare function clearCache(cache: Cache<unknown>, maxAge?: number): Promise<void>;
